#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Fichero: controlador.py
# Autor: Gonzalo Lezcano Hermoso
# Fecha de última modificación: 17 de marzo de 2018
#
#########################################################################
# Descripción: El siguiente es un fichero fundamental para la           #
# aplicación. Será el intermediario entre los distintos elementos       #
# de la aplicación.                                                     #
#########################################################################

# Importaciones generales
import os
import platform
import sys
import datetime

# Importaciones de PySNMP
from pysnmp.hlapi import *

# Importaciones de PyQt
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSql import *

# Para leer la tabla ARP
from python_arptable import get_arp_table

# Importaciones para lectura/escritura de config.xml
# cElemenTree es mucho más rápida y consume menos memoria que elemenTree
try:
    import xml.etree.cElementTree as ET
except importError:
    import xml.etree.ElementTree as ET

# Importaciones de nuestro proyecto
import snmpserver

__version__ = "1.0.3"
#########################################################################
#                       Historial de versiones                          #
# --------------------------------------------------------------------- #
# Versión 1.0.0: 6 de marzo de 2018                                     #
#                   Versión inicial                                     #
# Versión 1.0.1: 9 de marzo de 2018                                     #
#                   Cambio de máscara de subred a prefijo de subred     #
# Versión 1.0.2: 13 de marzo de 2018                                    #
#                   Inicialización de mibBuilder.                       #
#                   Desarrollo del método loadMib.                      #
#                   Desarrollo del método loadCompiler.                 #
#                   Carga inicial de módulos MIB.                       #
# Versión 1.0.3: 15 de marzo de 2018                                    #
#                   Apertura de conexión con la base de datos.          #
#                   Lectura/escritura en la base de datos para resync() #
#########################################################################


#########################################################################
#                       Clase: ControladorCentral                       #
#                                                                       #
# Esta clase proporciona los métodos y atributos necesarios para llevar #
# a cabo la labor de controlador central. Junto a la clase SnmpServer,  #
# presente en el fichero snmpserver.py, constituye el controlador en    #
# virtud de la programación Modelo/Vista/Controlador.                   #
#########################################################################
class ControladorCentral():

    def __init__(self):
        super(ControladorCentral, self).__init__()

        # Nos creamos una instancia de SnmpServer como parte del controlador
        self.miServidor = snmpserver.SnmpServer()

        # El objetivo es que, en un futuro, estos tres valores sean leídos de config.xml
        nombrebbdd = "tfg"
        usuariobbdd = "lezcano"
        passbbdd = "B4d5_28klo"

        # Cargamos e intentamos abrir la base de datos, operando con MySQL
        db = QSqlDatabase.addDatabase("QMYSQL")
        db.setDatabaseName(nombrebbdd)
        db.setUserName(usuariobbdd)
        db.setPassword(passbbdd)

        if not db.open():
            QMessageBox.warning(None, "Error de base de datos", db.lastError().text())
            sys.exit(1)


    #####################################################################
    #                       Método: resync                              #
    #                                                                   #
    # Este método se encargará de realizar un escaneo en la porción de  #
    # red indicada en config.xml. Para ello, se valdrá del método       #
    # snmpget, de la clase SnmpServer, preguntando en cada dirección    #
    # presente en el rango por el sysObjectID. Una vez descubiertos,    #
    # obtendrá algún dato adicional (dirección MAC y otros parámetros   #
    # vía SNMP) y escribirá en la base de datos, para la que se ha      #
    # abierto una conexión previamente. Este método es llamado desde    #
    # la ventana principal (clase MainWindow), a petición del usuario   #
    # cuando pulse sobre el icono correspondiente.                      #
    #####################################################################
    def resync(self):

        # Elegimos nuestro fichero de configuración
        tree = ET.parse('config.xml')

        # Extraemos de dicho fichero dirección, la máscara de subred y la community de lectura
        for elem in tree.iter():
            print(elem.tag, elem.text)
            if elem.tag == 'Subnet':
                subnetIpAddress = elem.text
            elif elem.tag == 'Prefix':
                self.subnetPrefix = int(elem.text)
            elif elem.tag == 'CommunityRO':
                communityRO = elem.text

        # Creamos las variables necesarias para el parseo de estas direcciones
        subnetIpByte1=''
        subnetIpByte2=''
        subnetIpByte3=''
        subnetIpByte4=''
        primerByteSI = False
        segundoByteSI = False
        tercerByteSI = False

        # Dividimos la dirección proporcionada en bytes
        for char in subnetIpAddress:
            if char is not '.' and primerByteSI is not True:
                subnetIpByte1 = subnetIpByte1 + char
            elif char == '.' and primerByteSI is not True:
                primerByteSI = True
            elif char is not '.' and segundoByteSI is not True:
                subnetIpByte2 = subnetIpByte2 + char
            elif char == '.' and segundoByteSI is not True:
                segundoByteSI = True
            elif char is not '.' and tercerByteSI is not True:
                subnetIpByte3 = subnetIpByte3 + char
            elif char == '.' and tercerByteSI is not True:
                tercerByteSI = True
            else:
                subnetIpByte4 = subnetIpByte4 + char

        # Hacemos que el resultado obtenido sea tratado como un entero
        subnetIpByte1 = int(subnetIpByte1)
        subnetIpByte2 = int(subnetIpByte2)
        subnetIpByte3 = int(subnetIpByte3)
        subnetIpByte4 = int(subnetIpByte4)

        # Transformamos el prefijo leído del XML a bytes de la máscara de subred
        if self.subnetPrefix > 8:
            subnetMaskByte1 = 255
            if self.subnetPrefix > 16:
                subnetMaskByte2 = 255
                if self.subnetPrefix > 24:
                    subnetMaskByte3 = 255
                    subnetMaskByte4 = self.ayudaMask(self.subnetPrefix - 24)
                else:
                    subnetMaskByte3 = self.ayudaMask(self.subnetPrefix - 16)
                    subnetMaskByte4 = 0
            else:
                subnetMaskByte2 = self.ayudaMask(self.subnetPrefix - 8)
                subnetMaskByte3 = 0
                subnetMaskByte4 = 0
        else:
            subnetMaskByte1 = self.ayudaMask(self.subnetPrefix)
            subnetMaskByte2 = 0
            subnetMaskByte3 = 0
            subnetMaskByte4 = 0

        # Preparamos las variables necesarias para obtener nuestro primer
        # y nuestro último ítem.
        subnetByte1 = subnetMaskByte1 & subnetIpByte1
        subnetByte2 = subnetMaskByte2 & subnetIpByte2
        subnetByte3 = subnetMaskByte3 & subnetIpByte3
        subnetByte4 = subnetMaskByte4 & subnetIpByte4
        primerByte = False
        segundoByte = False
        tercerByte = False
        primero = []
        ultimo = []

        # En función de la máscara de subred, encontramos dónde terminan los bits
        # a "1" y extraemos las direcciones de subred y broadcast. El primer
        # elemento a evaluar será el siguiente a la dirección de subred y el
        # último será el previo a la dirección de broadcast.
        if subnetMaskByte1 == 255:
            primero.append(subnetByte1)
            ultimo.append(subnetByte1)
            if subnetMaskByte2 == 255:
                primero.append(subnetByte2)
                ultimo.append(subnetByte2)
                if subnetMaskByte3 == 255:
                    primero.append(subnetByte3)
                    ultimo.append(subnetByte3)
                    # Seleccionamos como primero la dirección siguiente a la de subred
                    primero.append(subnetByte4 + 1)
                    # Seleccionamos como último la dirección previa a la de broadcast
                    ultimoItemByte4 = (((1 << 8) - 1 - subnetMaskByte4) | subnetIpByte4) - 1
                    ultimo.append(ultimoItemByte4)
                else:
                    primero.append(subnetByte3)
                    primero.append(subnetByte4 + 1)
                    ultimoItemByte3 = (((1 << 8) - 1 - subnetMaskByte3) | subnetIpByte3)
                    ultimo.append(ultimoItemByte3)
                    ultimo.append(254)
                    tercerByte = True
            else:
                primero.append(subnetByte2)
                primero.append(subnetByte3)
                primero.append(subnetByte4 + 1)
                ultimoItemByte2 = (((1 << 8) - 1 - subnetMaskByte2) | subnetIpByte2)
                ultimo.append(ultimoItemByte2)
                ultimo.append(255)
                ultimo.append(254)
                segundoByte = True
        else:
            primero.append(subnetByte1)
            primero.append(subnetByte2)
            primero.append(subnetByte3)
            primero.append(subnetByte4 + 1)
            ultimoItemByte1 = (((1 << 8) - 1 - subnetMaskByte1) | subnetIpByte1)
            ultimo.append(ultimoItemByte1)
            ultimo.append(255)
            ultimo.append(255)
            ultimo.append(254)
            primerByte = True

        # Agruparemos los elementos descubiertos como un diccionario, con clave su dirección IP
        # y valor su sysObjectID.
        elementos = {}

        # Este método genera una pérdida de 5 segundos por cada hueco vacío (por el timeout ARP)
        # De momento se encuentra únicamente desarrollado para subredes de clase C o menores
        # Realizamos un bucle entre el primer y el último elemento del rango a inspeccionar,
        # enviando un snmpget del sysObjectID para cada elemento presente en dicho rango.
        # Si obtenemos respuesta, guardamos la parte de la respuesta que se encuentre
        # tras el '=' (pues viene en formato SNMPv2-MIB::system-sysObjectID = valor),
        # damos formato adecuado para su IPv4 y lo almacenamos en nuestro diccionario de elementos
        if primerByte:
            pass
        elif segundoByte:
            pass
        elif tercerByte:
            pass
        else:
            i = primero[3]
            while i <= ultimo[3]:
                posibleElemento = str(primero[0]) + '.' + str(primero[1]) + '.' + str(primero[2]) + '.' + str(i)
                respuestaDiscover = self.miServidor.snmpget(".1.3.6.1.2.1.1.2.0", posibleElemento, communityRO)
                i = i+1
                if respuestaDiscover is not None:
                    respuestaDiscover = str(respuestaDiscover)
                    aGuardar = respuestaDiscover.split("= ")
                    aGuardar = aGuardar[1]
                    ipElemento = str(primero[0]) + '.' + str(primero[1]) + '.' + str(primero[2]) + '.' + str(i-1)
                    elementos[ipElemento] = [respuestaDiscover]

        # Hacemos el walk para los elementos descubiertos y preparamos la consulta a la BBDD
        for elemento in elementos:
            diccionarioQuery = {"mac_address":'', "ip_address":elemento, "lastTimeDetected":datetime.datetime.today(),
                                "sysObjectID":aGuardar, "sysDescr":'', "sysUpTime":'', "sysContact":'',
                                "sysName":'', "sysLocation":'', "sysServices":''}
            print("Elemento: " + elemento)
            print("sysObjectID: " + diccionarioQuery["sysObjectID"])
            miRespuesta = self.miServidor.snmpgetnext('1.3.6.1.2.1.1', elemento, communityRO)
            for varBindTableRow in miRespuesta:
                for name, val in varBindTableRow:
                    if 'sysDescr.0' in name.prettyPrint():
                        diccionarioQuery["sysDescr"] = val.prettyPrint()
                        print("sysDescr: " + diccionarioQuery["sysDescr"])
                    elif 'sysUpTime.0' in name.prettyPrint():
                        diccionarioQuery["sysUpTime"] = val.prettyPrint()
                        print("sysUpTime: " + diccionarioQuery["sysUpTime"])
                    elif 'sysContact.0' in name.prettyPrint():
                        diccionarioQuery["sysContact"] = val.prettyPrint()
                        print("sysContact: " + diccionarioQuery["sysContact"])
                    elif 'sysName.0' in name.prettyPrint():
                        diccionarioQuery["sysName"] = val.prettyPrint()
                        print("sysName: " + diccionarioQuery["sysName"])
                    elif 'sysUpTime.0' in name.prettyPrint():
                        diccionarioQuery["sysUpTime"] = val.prettyPrint()
                        print("sysUpTime: " + diccionarioQuery["sysUpTime"])
                    elif 'sysLocation.0' in name.prettyPrint():
                        diccionarioQuery["sysLocation"] = val.prettyPrint()
                        print("sysLocation: " + diccionarioQuery["sysLocation"])
                    elif 'sysServices.0' in name.prettyPrint():
                        diccionarioQuery["sysServices"] = val.prettyPrint()
                        print("sysServices: " + diccionarioQuery["sysServices"])

            # Obtenemos la tabla ARP para, relacionar la dirección MAC con la IP del elemento descubierto
            # y la consultamos para enviarla en la consulta SQL.
            tablaArp = get_arp_table()
            for entrada in tablaArp:
                print("Entrada: " + str(entrada))
                print("IP: " + str(entrada['IP address']))
                print("MAC: " + str(entrada['HW address']))
                if entrada['IP address'] == elemento:
                    diccionarioQuery["mac_address"] = entrada['HW address']

            # Creamos una instancia QSqlQuery para realizar nuestra petición
            petiSql = QSqlQuery()

            # Preparamos una actualización si la MAC del elemento ya estaba en la BBDD.
            # Podría darse un error de MAC duplicada que no captáramos, pero, dado que su aparición sucede
            # con poca frecuencia, lo dejamos fuera del alcance de nuestra aplicación, siendo el usuario el que
            # deberá proporcionar los adecuados métodos para evitar este problema.
            peticion1 = "UPDATE elementos SET ip_address='" + diccionarioQuery["ip_address"] +\
                        "', lastTimeDetected='" + str(diccionarioQuery["lastTimeDetected"]) +\
                        "', sysObjectID='" + diccionarioQuery["sysObjectID"] +\
                        "', sysDescr='" + diccionarioQuery["sysDescr"] +\
                        "', sysUpTime='" + str(diccionarioQuery["sysUpTime"]) +\
                        "', sysContact='" + diccionarioQuery["sysContact"] +\
                        "', sysName='" + diccionarioQuery["sysName"] +\
                        "', sysLocation='" + diccionarioQuery["sysLocation"] +\
                        "', sysServices='" + diccionarioQuery["sysServices"] +\
                        "' WHERE mac_address='"+ diccionarioQuery["mac_address"] + "';"

            # Ejecutamos la petición previamente preparada
            petiSql.exec_(peticion1)

            # Si no existía ningún elemento con esa MAC, no habremos escrito nada en la BBDD y, por lo tanto,
            # preparamos la consulta de inserción con los datos presentes en nuestro diccionario.
            if petiSql.numRowsAffected() == 0:
                peticion2 = "INSERT INTO elementos VALUES ('" + diccionarioQuery["mac_address"] + "', '" + \
                diccionarioQuery["ip_address"] + "', '" + str(diccionarioQuery["lastTimeDetected"]) + "', '" + \
                diccionarioQuery["sysObjectID"] + "', '" + diccionarioQuery["sysDescr"] + "', '" + \
                str(diccionarioQuery["sysUpTime"]) + "', '" + diccionarioQuery["sysContact"] + "', '" + \
                diccionarioQuery["sysName"] + "', '" + diccionarioQuery["sysLocation"] + "', '" + \
                diccionarioQuery["sysServices"] + "');"

                # Ejecutamos la petición previamente preparada
                petiSql.exec_(peticion2)


    #####################################################################
    #                       Método: ayudaMask                           #
    #                                                                   #
    # El cometido de este método es devolver el valor de un byte de la  #
    # máscara de subred, proporcionado el número de bits a 1 en dicho   #
    # octeto. Es llamado desde el método resync de la misma clase.      #
    #####################################################################
    def ayudaMask(self, num):
        salidaMask = 0
        auxiliar = num
        auxiliar2 = 7
        while auxiliar > 0:
            salidaMask = salidaMask + 2 ** auxiliar2
            auxiliar -= 1
            auxiliar2 -= 1
        return salidaMask


    #####################################################################
    #                       Método: loadMib                             #
    #                                                                   #
    # Método destinado a ejercer la conexión entre la llamada desde     #
    # LoadMibDlgModule.accept() y el método loadMib de SnmpServer.      #
    #####################################################################
    def loadMib(self, *args):
        self.miServidor.loadMib(*args)


    #####################################################################
    #                       Método: loadCompiler                        #
    #                                                                   #
    # Método destinado a ejercer la conexión entre la llamada desde     #
    # LoadMibDlgCompiler.accept() y el método loadCompiler de           #
    # SnmpServer.                                                       #
    #####################################################################
    def loadCompiler(self, arg):
        self.miServidor.loadCompiler(arg)