#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Fichero: editarConf.py
# Autor: Gonzalo Lezcano Hermoso
# Fecha de última modificación: 16 de marzo de 2018
#
#########################################################################
# Descripción: El objetivo del siguiente fichero es presentar un        #
# diálogo para alterar la configuración (contenida en el fichero        #
# config.xml) cuando el usuario así lo seleccione, clickando sobre el   #
# icono presente en la barra de herramientas, destinado a alterar la    #
# configuración.                                                        #
#########################################################################

# Importaciones generales
import os
import platform
import sys

# Importaciones de PyQT
from PyQt4.QtCore import *
from PyQt4.QtGui import *

# Importaciones para lectura/escritura de config.xml
# cElemenTree es mucho más rápida y consume menos memoria que elemenTree
try:
    import xml.etree.cElementTree as ET
except importError:
    import xml.etree.ElementTree as ET

__version__ = "1.0.1"
#########################################################################
#                       Historial de versiones                          #
# --------------------------------------------------------------------- #
# Versión 1.0.0: 8 de marzo de 2018                                     #
#                   Versión inicial.                                    #
# Versión 1.0.1: 16 de marzo de 2018                                    #
#                   Adición de la carga de MIB como opción configurable.#
#########################################################################


#########################################################################
#                       Clase: EditarConfDlg                            #
#                                                                       #
# La siguiente clase proporciona los métodos y atributos necesarios     #
# para dibujar el diálogo destinado a editar la configuración y su      #
# comunicación con el fichero de configuración (config.xml) a fin de    #
# escribir los cambios indicados por el usuario.                        #
#########################################################################
class EditarConfDlg(QDialog):

    # Inicialización. Abriremos el fichero de configuración, prepararemos
    # una lista para almacenar los módulos MIB a cargar (cuyo número es
    # desconocido), llamaremos a nuestro método particular para cargar el
    # diálogo adecuado y estableceremos el título de la ventana emergente.
    def __init__(self, parent=None):
        super(EditarConfDlg, self).__init__(parent)

        self.tree = ET.parse('config.xml')
        self.root = self.tree.getroot()
        self.mibUpload = []
        self.cargarDialogo()
        self.setWindowTitle("Modificar la configuración")


    #####################################################################
    #                       Método: cargarDialogo                       #
    #                                                                   #
    # Cargará el diálogo principal para editar la configuración. Será   #
    # llamado al inicio de una instanciación de nuestra clase.          #
    #####################################################################
    def cargarDialogo(self):

        # Recorremos el fichero config.xml y extraemos los parámetros relevantes.
        for elem in self.tree.iter():
            print(elem.tag, elem.text)
            if elem.tag == 'Subnet':
                self.subnetIp = elem.text
            elif elem.tag == 'Prefix':
                self.subnetPrefix = elem.text
            elif elem.tag == 'CommunityRO':
                self.communityRO = elem.text
            elif elem.tag == 'CommunityRW':
                self.communityRW = elem.text
            elif elem.tag == 'MibUpload':
                for child in elem:
                    self.mibUpload.append(child.text)

        # Etiquetas del diálogo
        etiqueta1 = QLabel("IP subred")
        etiqueta2 = QLabel("Máscara de subred")
        etiqueta3 = QLabel("Community Read-Only")
        etiqueta4 = QLabel("Community Read-Write")
        etiqueta5 = QLabel("Módulos MIB cargados al arranque")

        # Celdas del diálogo. Inicialmente presentarán los valores leídos
        # del fichero config.xml
        self.subnetIpField = QLineEdit(self.subnetIp)
        self.subnetPrefixField = QLineEdit(self.subnetPrefix)
        self.communityROField = QLineEdit(self.communityRO)
        self.communityRWField = QLineEdit(self.communityRW)

        # Botón "Añadir", para añadir más módulos MIB
        addButton = QPushButton("Añadir módulo")

        # Botones "Aplicar | Cerrar"
        buttonBox = QDialogButtonBox(QDialogButtonBox.Apply|QDialogButtonBox.Close)

        # Dibujamos el diálogo. Al no estar cerrado el número de módulos MIB
        # posibles, realizamos un bucle y dibujamos estos elementos y los posteriores
        # en función de un contador (i).
        grid = QGridLayout()
        grid.addWidget(etiqueta1, 0, 0)
        grid.addWidget(self.subnetIpField, 0, 1)
        grid.addWidget(etiqueta2, 1, 0)
        grid.addWidget(self.subnetPrefixField, 1, 1)
        grid.addWidget(etiqueta3, 2, 0)
        grid.addWidget(self.communityROField, 2, 1)
        grid.addWidget(etiqueta4, 3, 0)
        grid.addWidget(self.communityRWField, 3, 1)
        grid.addWidget(etiqueta5, 4, 0, 1, 2)
        i = 0
        self.listaCeldasMib = []
        for item in self.mibUpload:
            self.celdaTemporal = QLineEdit(self.mibUpload[i])
            self.listaCeldasMib.insert(i, self.celdaTemporal)
            grid.addWidget(self.celdaTemporal, i+5, 0, 1, 2)
            i += 1
        grid.addWidget(addButton, i+5, 1)
        grid.addWidget(buttonBox, i+6, 0, 1, 2)
        self.setLayout(grid)

        # Conectamos los botones a las señales apropiadas
        self.connect(buttonBox.button(QDialogButtonBox.Apply), SIGNAL("clicked()"), self.apply)
        self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))
        self.connect(addButton, SIGNAL("clicked()"), self.addField)


    #####################################################################
    #                           Método: apply                           #
    #                                                                   #
    # Su cometido es el de recoger los valores introducidos por el      #
    # usuario, comprobar los campos sensibles y escribir, en caso de    #
    # que el resultado de las comprobaciones sea exitoso, los cambios   #
    # correspondientes en el fichero config.xml                         #
    #####################################################################
    def apply(self):

        # Recogemos los valores de los distintos campos
        # ESTO HAY QUE MODIFICARLO POR RAZONES DE EFICIENCIA Y QUE SE HAGA REFERENCIA SIEMPRE AL FIELD.TEXT()
        self.subnetIp = self.subnetIpField.text()
        self.subnetPrefix = self.subnetPrefixField.text()
        self.communityRO = self.communityROField.text()
        self.communityRW = self.communityRWField.text()



        # Ordenamos las comprobaciones de corrección apropiadas. En caso de que no satisfaga
        # los requerimientos será retornado un 1 por los métodos llamados.
        if self.comprobarIp() == 1:
            return
        if self.comprobarPrefix() == 1:
            return

        # Preparamos la escritura en nuestro fichero de configuración. Si alguno de los campos
        # destinados a los módulos MIB se encuentra vacío, se eliminará del fichero de configuración.
        for elem in self.tree.iter():
            print(elem.tag, elem.text)
            if elem.tag == 'Subnet':
                elem.text = self.subnetIp
            elif elem.tag == 'Prefix':
                elem.text = str(self.subnetPrefix)
            elif elem.tag == 'CommunityRO':
                elem.text = self.communityRO
            elif elem.tag == 'CommunityRW':
                elem.text = self.communityRW
            elif elem.tag == 'MibUpload':
                i = 0
                for child in elem:
                    if self.listaCeldasMib[i].text() is '':
                        elem.remove(child)
                    else:
                        child.text = self.listaCeldasMib[i].text()
                        print(child.text)
                    i += 1

        # Escribimos los cambios apropiados en el fichero de configuración y cerramos la ventana.
        try:
            self.tree.write('config.xml')
            QMessageBox.information(self, "Configuración modificada", "Sus cambios han sido correctamente escritos en el fichero de configuración.")
            self.close()
        except Exception:
            QMessageBox.warning(self, "Error", "Su configuración no ha sido modificada debido a algún error.")
            self.close()


    #####################################################################
    #                       Método: addField                            #
    #                                                                   #
    # Este método crea una instancia del diálogo destinado a añadir un  #
    # nuevo módulo MIB al fichero de configuración y ejecutarlo. Una    #
    # vez finalizada la ejecución de este diálogo, cerrará el principal.#
    # Si el usuario volviese a abrir el diálogo editConfDlg, podría ver #
    # el módulo recién insertado en el fichero config.xml               #
    #####################################################################
    def addField(self):
        miAdicion = AddModuleDlg()
        miAdicion.exec_()
        self.close()


    #####################################################################
    #                       Método: comprobarIp                         #
    #                                                                   #
    # El objetivo de este método es determinar si el valor introducido  #
    # por el usuario tiene el formato correcto. Se esperan direcciones  #
    # IPv4. Se esperan por tanto 4 bytes en formato decimal separados   #
    # por puntos ('.'), sin valores negativos y, por lo tanto, con un   #
    # valor máximo de 255 (2⁸ - 1) por octeto. Que un usuario           #
    # introduzca un valor que, conforme al prefijo, no corresponda a    #
    # una dirección de subred (por ejemplo, 192.168.100.130/24), no se  #
    # comprobará en este método y se realizará la conversión a una      #
    # dirección de subred correcta en el método resync de la clase      #
    # ControladorCentral.                                               #
    #####################################################################
    def comprobarIp(self):

        # Inicializamos nuestras variables de control
        contadorBytes = 0
        contadorDigitos = 0
        octeto = ''

        # El campo no puede estar vacío
        if self.subnetIp is '':
            QMessageBox.warning(self, "Error de formato", "El campo de dirección IP no puede quedar vacío.")
            self.subnetIpField.setFocus()
            return 1

        # Si el campo no está vacío, lo recorremos caracter a caracter para determinar su corrección
        for char in self.subnetIp:
            # Si el caracter no es un punto, deberíamos encontrarnos dentro de un byte
            if char is not ".":
                # Comprobamos que, de no tratarse de un '.', es un caracter numérico
                if char not in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']:
                    QMessageBox.warning(self, "Error de formato", "La dirección IP y la máscara deben"
                                                                  " ser escritas en formato decimal, con un"
                                                                  " máximo de 3 dígitos por byte")
                    self.subnetIpField.selectAll()
                    self.subnetIpField.setFocus()
                    return 1
                contadorDigitos += 1

                # Comprobamos que no se han introducido más de 3 dígitos en el byte
                if contadorDigitos > 3:
                    QMessageBox.warning(self, "Error de formato", "La dirección IP y la máscara deben"
                                                                  " ser escritas en formato decimal, con un"
                                                                  " máximo de 3 dígitos por byte.")
                    self.subnetIpField.selectAll()
                    self.subnetIpField.setFocus()
                    return 1
                octeto = octeto + char

                # Comprobamos que el valor del byte no es superior a 255
                if contadorDigitos == 3:
                    if int(octeto) > 255:
                        QMessageBox.warning(self, "Error de formato", "Cada byte debe tener un valor menor o igual"
                                                                      " a 255.")
                        self.subnetIpField.selectAll()
                        self.subnetIpField.setFocus()
                        return 1

            # Si el caracter es un punto nos encontramos ante una separación de bytes
            else:
                # Comprobamos si el byte anterior ha quedado vacío
                if contadorDigitos == 0:
                    QMessageBox.warning(self, "Error de formato", "No puede haber ningún octeto"
                                                                  " vacío. Ni en el campo de dirección IP ni en el de máscara"
                                                                  " de subred.")
                    self.subnetIpField.selectAll()
                    self.subnetIpField.setFocus()
                    return 1

                else:
                    # Si no ha quedado vacío, comprobamos que no es mayor de 255
                    if int(octeto) > 255:
                        QMessageBox.warning(self, "Error de formato", "Cada byte debe tener un valor menor o igual"
                                                                      " a 255.")
                        self.subnetIpField.selectAll()
                        self.subnetIpField.setFocus()
                        return 1

                    # Si ha ido bien, incrementamos el contador de bytes, ponemos a 0 el contador de dígitos
                    # y dejamos vacía la cadena octeto.
                    contadorBytes += 1
                    contadorDigitos = 0
                    octeto = ""

                # Si el contador de bytes supera el 3 significará que se han introducido más de 4 bytes
                if contadorBytes > 3:
                    QMessageBox.warning(self, "Error de formato", "La dirección IP y la máscara de subred"
                                                                  " deben tener 4 bytes.")
                    self.subnetIpField.selectAll()
                    self.subnetIpField.setFocus()
                    return 1


    #####################################################################
    #                   Método: comprobarPrefix                         #
    #                                                                   #
    # El objetivo de este método es determinar si el valor introducido  #
    # por el usuario tiene el formato correcto. Se esperan prefijos     #
    # enteros entre 0 y 32. Que un usuario introduzca un valor que,     #
    # conforme a la dirección que se haya escrito, no corresponda a     #
    # una dirección de subred (por ejemplo, 192.168.100.130/24), no se  #
    # comprobará en este método y se realizará la conversión a una      #
    # dirección de subred correcta en el método resync de la clase      #
    # ControladorCentral.                                               #
    #####################################################################
    def comprobarPrefix(self):

        # Si el valor no es un entero recogeremos la excepción
        try:
            self.subnetPrefix = int(self.subnetPrefixField.text())
        except ValueError:
            QMessageBox.warning(self, "Error de formato", "El prefijo de subred debe ser un entero.")
            self.subnetPrefixField.selectAll()
            self.subnetPrefixField.setFocus()
            return 1

        # No es aceptable un valor por encima de 32
        if self.subnetPrefix > 32:
            QMessageBox.warning(self, "Error de formato", "La máscara puede tener un máximo de 32 bits a '1'.")
            self.subnetPrefixField.selectAll()
            self.subnetPrefixField.setFocus()
            return 1

        # No es aceptable un valor negativo
        elif self.subnetPrefix < 0:
            QMessageBox.warning(self, "Error de formato", "La máscara no puede tener un número negativo de bits a '1'.")
            self.subnetPrefixField.selectAll()
            self.subnetPrefixField.setFocus()
            return 1


#########################################################################
#                       Clase: AddModuleDlg                             #
#                                                                       #
# La siguiente clase proporciona los métodos y atributos necesarios     #
# para dibujar el diálogo destinado a insertar un nuevo módulo MIB en   #
# el fichero config.xml. Este diálogo será llamado únicamente desde el  #
# diálogo editConfDlg.                                                  #
#########################################################################
class AddModuleDlg(QDialog):

    def __init__(self, parent=None):
        super(AddModuleDlg, self).__init__(parent)

        # Elementos que conforman el diálogo
        etiqueta = QLabel("¿Qué módulo MIB quiere incluir?")
        self.insercionModule = QLineEdit("Escriba aquí el módulo a incluir...")
        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)

        # Dibujo del Layout
        grid = QVBoxLayout()
        grid.addWidget(etiqueta, 0)
        grid.addWidget(self.insercionModule, 1)
        grid.addWidget(buttonBox, 2)
        self.setLayout(grid)
        self.setWindowTitle("Añadir nuevo campo MIB")

        # Dejamos el control sobre el campo de inserción
        self.insercionModule.selectAll()
        self.insercionModule.setFocus()

        # Conexiones
        self.connect(buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
        self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))


    #####################################################################
    #                       Método: accept                              #
    #                                                                   #
    # Este módulo tiene como objetivo introducir en el fichero de       #
    # configuración el módulo deseado por el usuario. Para ello,        #
    # generará la etiqueta adecuada e insertará el subelemento a        #
    # continuación de los módulos MIB existentes en el fichero de       #
    # configuración en ese momento.                                     #
    #####################################################################
    def accept(self):
        try:
            tree = ET.parse('config.xml')
            for elem in tree.iter():
                if elem.tag == 'MibUpload':
                    i = 1
                    for child in elem:
                        i += 1
                    etiquetaXml = 'Mib' + str(i)
                    nuevaMib = ET.Element(etiquetaXml)
                    nuevaMib.text = self.insercionModule.text()
                    elem.append(nuevaMib)
                tree.write('config.xml')
            QMessageBox.information(self, "¡Conseguido!", "Tus módulos se han cargado satisfactoriamente.")
        except Exception:
            QMessageBox.warning(self, "Error", "Tu petición no ha podido ser atendida satisfactoriamente.")


# Hacemos que el diálogo se ejecute si es llamado desde la función main(), presente en ventanaPrincipal.pyw
if __name__ == "__main__":
    app = QApplication(sys.argv)
    form = EditarConfDlg()
    form.show()
    app.exec_()