#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Fichero: loadMib.py
# Autor: Gonzalo Lezcano Hermoso
# Fecha de última modificación: 17 de marzo de 2018
#
#########################################################################
# Descripción: El objetivo del siguiente fichero es presentar un        #
# diálogo a fin de cargar distintos módulos MIB no cargados previamente.#
# Se presentará un diálogo inicial en el que el usuario decidirá si     #
# cargar un módulo (presente en los compiladores existentes) o cargar   #
# un compilador. En este mismo fichero se encuentran los diálogos para  #
# ambas opciones y el código necesario para su carga.                   #
#########################################################################

# Importaciones generales
import os
import platform
import sys

# Importaciones de PyQT
from PyQt4.QtCore import *
from PyQt4.QtGui import *

# Importaciones de PySNMP
from pysnmp.smi import builder, view, compiler, error
from pysnmp.hlapi import *

__version__ = "1.0.0"
#########################################################################
#                       Historial de versiones                          #
# --------------------------------------------------------------------- #
# Versión 1.0.0: 12 de marzo de 2018                                    #
#                   Versión inicial.                                    #
#########################################################################


#########################################################################
#                       Clase: LoadMibDlgIni                            #
#                                                                       #
# Esta clase sirve para crear un diálogo destinado a la carga de nuevos #
# módulos o compiladores MIB. Es instanciada desde la ventana principal #
# a petición del usuario, cuando clicke sobre el icono dispuesto a tal  #
# fin en la barra de herramientas.                                      #
#########################################################################
class LoadMibDlgIni(QDialog):

    def __init__(self, instanciaControlador, parent=None):
        super(LoadMibDlgIni, self).__init__(parent)

        # Instancia del controlador para indicarle las cargas
        self.instanciaControlador = instanciaControlador

        # Elementos que conforman el diálogo
        etiqueta = QLabel("¿Quiere cargar un módulo o un compilador?")
        moduleButton = QPushButton("Cargar módulo")
        compiladorButton = QPushButton("Cargar compilador")

        # Dibujo del Layout
        grid = QGridLayout()
        grid.addWidget(etiqueta, 0, 0, 1, 2)
        grid.addWidget(moduleButton, 1, 0)
        grid.addWidget(compiladorButton, 1, 1)
        self.setLayout(grid)

        # Conexiones
        self.connect(moduleButton, SIGNAL("clicked()"), self.conectarModulo)
        self.connect(compiladorButton, SIGNAL("clicked()"), self.conectarCompilador)


    #####################################################################
    #                   Método: conectarModulo                          #
    #                                                                   #
    # Método destinado a lanzar el diálogo LoadMibDlgModule.            #
    #####################################################################
    def conectarModulo(self):
        dialog = LoadMibDlgModule(self.instanciaControlador)
        dialog.exec_()


    #####################################################################
    #                   Método: conectarCompilador                      #
    #                                                                   #
    # Método destinado a lanzar el diálogo LoadMibDlgCompiler.          #
    #####################################################################
    def conectarCompilador(self):
        dialog = LoadMibDlgCompiler(self.instanciaControlador)
        dialog.exec_()


#########################################################################
#                       Clase: LoadMibDlgModule                         #
#                                                                       #
# La presente clase representa el diálogo lanzado para cargar un nuevo  #
# módulo MIB de manera no persistente. Es instanciada desde             #
# LoadMibDlgIni a petición del usuario, tras pulsar en el botón         #
# correspondiente de dicho diálogo.                                     #
#########################################################################
class LoadMibDlgModule(QDialog):

    # Inicialización, con la instancia de ControladorCentral pasada por LoadMibDlgIni
    def __init__(self, instanciaControlador, parent=None):
        super(LoadMibDlgModule, self).__init__(parent)

        self.instanciaControlador = instanciaControlador

        # Elementos que conforman el diálogo
        etiqueta = QLabel("¿Qué módulo MIB quiere cargar?")
        self.insercionModule = QLineEdit("Escriba aquí el módulo a cargar...")
        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)

        # Dibujo del Layout
        grid = QVBoxLayout()
        grid.addWidget(etiqueta, 0)
        grid.addWidget(self.insercionModule, 1)
        grid.addWidget(buttonBox, 2)
        self.setLayout(grid)

        # Dejamos el control sobre el campo de inserción
        self.insercionModule.selectAll()
        self.insercionModule.setFocus()

        # Conexiones
        self.connect(buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
        self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))


    #####################################################################
    #                       Método: accept                              #
    #                                                                   #
    # Método destinado a cargar el módulo MIB que el usuario haya       #
    # introducido a través del método loadMib del controlador central.  #
    #####################################################################
    def accept(self):
        try:
            self.instanciaControlador.loadMib(str(self.insercionModule))
            QMessageBox.information(self, "¡Conseguido!", "Tus módulos se han cargado satisfactoriamente.")
        except error:
            QMessageBox.warning(self, "Error", "Tu petición no ha podido ser atendida satisfactoriamente.")


#########################################################################
#                       Clase: LoadMibDlgCompiler                       #
#                                                                       #
# La presente clase representa el diálogo lanzado para cargar un nuevo  #
# compilador MIB de manera no persistente. Es instanciada desde         #
# LoadMibDlgIni a petición del usuario, tras pulsar en el botón         #
# correspondiente de dicho diálogo.                                     #
#########################################################################
class LoadMibDlgCompiler(QDialog):

    def __init__(self, instanciaControlador, parent=None):
        super(LoadMibDlgCompiler, self).__init__(parent)

        # Inicialización, con la instancia de ControladorCentral pasada por LoadMibDlgIni
        self.instanciaControlador = instanciaControlador

        # Elementos que conforman el diálogo
        etiqueta = QLabel("¿Qué compilador MIB quiere cargar?")
        self.insercionCompiler = QLineEdit("Escriba aquí la ruta del módulo a cargar...")
        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)

        # Dibujo del layout
        grid = QVBoxLayout()
        grid.addWidget(etiqueta, 0)
        grid.addWidget(self.insercionCompiler, 1)
        grid.addWidget(buttonBox, 2)
        self.setLayout(grid)

        # Dejamos el control sobre el campo de inserción
        self.insercionCompiler.selectAll()
        self.insercionCompiler.setFocus()

        # Conexiones
        self.connect(buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
        self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))

    #####################################################################
    #                       Método: accept                              #
    #                                                                   #
    # Método destinado a cargar el compilador MIB que el usuario haya   #
    # introducido a través del método loadCompiler del controlador      #
    # central.                                                          #
    #####################################################################
    def accept(self):
        try:
            self.instanciaControlador.loadCompiler(str(self.insercionCompiler))
            QMessageBox.information(self, "¡Conseguido!", "Tus fuentes se han cargado satisfactoriamente.")
        except error.SmiError:
            QMessageBox.warning(self, "Error", "Tu petición no ha podido ser atendida satisfactoriamente.")