#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Fichero: snmpserver.py
# Autor: Gonzalo Lezcano Hermoso
# Fecha de última modificación: 14 de marzo de 2018
#
#########################################################################
# Descripción: El objetivo del siguiente fichero es representar la      #
# ventana principal del Trabajo Fin de Grado "Herramienta gráfica de    #
# gestión SNMP y manejo de MIB". En ésta, incluiremos un Toolbar con    #
# cinco distintas opciones y un widget central con dos ventanas que     #
# albergarán dos listas distintas: una para mostrar los elementos       #
# gestionados y la otra para mostrar las alarmas.                       #
#########################################################################

# Importaciones generales
import os
import platform
import sys

# Importaciones de PySNMP
from pysnmp.hlapi import *
from pysnmp.smi import builder, view, compiler, error
from pysnmp.entity.rfc3413.oneliner import cmdgen
from pysnmp.proto import rfc1902

# Importaciones para lectura/escritura de config.xml
# cElemenTree es mucho más rápida y consume menos memoria que elemenTree
try:
    import xml.etree.cElementTree as ET
except importError:
    import xml.etree.ElementTree as ET

# Importaciones de nuestro proyecto


# Versión
__version__ = "1.0.2"
#########################################################################
#                       Historial de versiones                          #
# --------------------------------------------------------------------- #
# Versión 1.0.0: 5 de marzo de 2018                                     #
#                   Métodos snmpget y snmpwalk.                         #
# Versión 1.0.1: 9 de marzo de 2018                                     #
#                   Método snmpset.                                     #
# Versión 1.0.2: 14 de marzo de 2018                                    #
#                   Los métodos loadMib y loadCompiler pasan a ser      #
#                   parte de la clase SnmpServer                        #
#########################################################################

#########################################################################
#                         Clase: SnmpServer                             #
#                                                                       #
# Agrupará las funcionalidades necesarias para realizar la función de   #
# servidor SNMP en nuestra aplicación, a saber, get, walk, set y        #
# trap-receiver.                                                        #
#########################################################################
class SnmpServer():

    def __init__(self):
        super(SnmpServer, self).__init__()

        self.mibBuilder = builder.MibBuilder()
        self.mibViewController = view.MibViewController(self.mibBuilder)

        tree = ET.parse('config.xml')

        # Leemos los módulos MIB a cargar inicialmente, y llamamos a loadMib para que los cargue
        mibUpload = []
        for elem in tree.iter(tag='MibUpload'):
            mibUpload.append(elem.text)

        self.loadMib(*mibUpload)


    #####################################################################
    #           Método: snmpget(consulta, direccion, community)         #
    #                                                                   #
    # Se encarga de realizar un get, tomando como argumentos el OID     #
    # objetivo (consulta), la dirección del agente objetivo (direccion) #
    # y la comunidad, que en este caso debería ser de sólo lectura.     #
    #####################################################################
    def snmpget(self, consulta, direccion, community):
        g = getCmd(SnmpEngine(),
                   CommunityData(community),
                   UdpTransportTarget((direccion, 161)),
                   ContextData(),
                   ObjectType(ObjectIdentity(consulta)))
        errorIndication, errorStatus, errorIndex, varBinds = next(g)

        if errorIndication:
            print(errorIndication)
        elif errorStatus:
            print('%s at %s' % (errorStatus.prettyPrint(), errorIndex and varBinds[int(errorIndex) - 1][0] or '?'))
        else:
            for varBind in varBinds:
                print(' = '.join([x.prettyPrint() for x in varBind]))
                return varBind


    def snmpwalk(self, direccion, community, argumento1):
        for (errorIndication,
             errorStatus,
             errorIndex,
             varBinds) in nextCmd(SnmpEngine(), CommunityData(community), UdpTransportTarget((direccion, 161)),
                                  ContextData(), ObjectType(ObjectIdentity(argumento1))):
            if errorIndication:
                print(errorIndication)
                break
            elif errorStatus:
                print('%s at %s' % (errorStatus.prettyPrint(), errorIndex and varBinds[int(errorIndex) - 1][0] or '?'))
            else:
                for varBind in varBinds:
                    print(' = '.join([x.prettyPrint() for x in varBind]))

    def snmpgetnext(self, argumento, direccion, community):
        cmdGen = cmdgen.CommandGenerator()

        errorIndication, errorStatus, errorIndex, varBindTable = cmdGen.nextCmd(
            cmdgen.CommunityData(community),
            cmdgen.UdpTransportTarget((direccion, 161)),
            argumento,
        )

        if errorIndication:
            print(errorIndication)
        else:
            if errorStatus:
                print('%s at %s' % (
                    errorStatus.prettyPrint(),
                    errorIndex and varBindTable[-1][int(errorIndex) - 1] or '?'
                )
                      )
            else:
                for varBindTableRow in varBindTable:
                    for name, val in varBindTableRow:
                        print('%s = %s' % (name.prettyPrint(), val.prettyPrint()))
                return varBindTable

    def snmpset(self, argumento, direccion, community, nuevoValor):
        errorIndication, errorStatus, errorIndex, varBinds = next(
            setCmd(SnmpEngine(),
                   CommunityData(community),
                   UdpTransportTarget((direccion, 161)),
                   ContextData(),
                   ObjectType(ObjectIdentity(argumento), nuevoValor)))

        if errorIndication:
            print(errorIndication)
        elif errorStatus:
            print('%s at %s' % (errorStatus.prettyPrint(),
                                errorIndex and varBinds[int(errorIndex) - 1][0] or '?'))
        else:
            for varBind in varBinds:
                print(' = '.join([x.prettyPrint() for x in varBind]))

    def decodificarOID(self, OID):
        #OID.resolveWithMib(mibViewController)
        print(OID.getMibSymbol())

    def loadMib(self, *args):
        print(builder.MibBuilder().getMibPath())
        try:
            self.mibBuilder.loadModules(*args)
            self.mibViewController = view.MibViewController(self.mibBuilder)
            print("Hola. Lo he conseguido")
        except error.SmiError:
            QMessageBox.warning(self, "Error SMI", "Alguna de las MIB pasadas para cargar, no ha podido ser cargada.")

    def loadCompiler(self, arg):
        try:
            compiler.addMibCompiler(self.mibBuilder, sources=arg)
            print("Hola. Lo he conseguido")
        except error.SmiError:
            QMessageBox.warning(self, "Error SMI", "Alguna de las MIB pasadas para cargar, no ha podido ser cargada.")


# Para ir haciendo las comprobaciones oportunas
#miPrueba = SnmpServer()
#miPrueba.snmpwalk('192.168.1.13', 'public', '1.3.6.1.2.1.1')
#miPrueba.voyAProbar()
#miPrueba.voyAProbar2()