#!/usr/bin/env python3.5
# -*- coding: utf-8 -*-
# Fichero: ventanaPrincipal.pyw
# Autor: Gonzalo Lezcano Hermoso
# Fecha de última modificación: 15 de marzo de 2018
#
#########################################################################
# Descripción: El objetivo del siguiente fichero es representar la      #
# ventana principal del Trabajo Fin de Grado "Herramienta gráfica de    #
# gestión SNMP y manejo de MIB". En ésta, incluiremos un Toolbar con    #
# cinco distintas opciones y un widget central con dos ventanas que     #
# albergarán dos listas distintas: una para mostrar los elementos       #
# gestionados y la otra para mostrar las alarmas.                       #
#########################################################################

# Importaciones generales
import os
import platform
import sys

# Importaciones de PyQT
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSql import *

# Importaciones de nuestra aplicación
import qrc_recursos # Proporcionamos una ruta inequívoca para nuestros recursos
import controlador # Llamadas al controlador central
import editarConf # Para llamar al diálogo destinado a modificar la configuración
import loadMib # Para realizar las cargas de MIB

__version__ = "1.0.3"
#########################################################################
#                       Historial de versiones                          #
# --------------------------------------------------------------------- #
# Versión 1.0.0: 5 de marzo de 2018                                     #
#                   Interfaz gráfica inicial                            #
# Versión 1.0.1: 9 de marzo de 2018                                     #
#                   Adición del formato tabla para el centralWidget.    #
#                   Conexión con la opción de modificarConf.            #
# Versión 1.0.2: 12 de marzo de 2018                                    #
#                   Conexión con la opción de carga de MIBs.            #
#                   Carga inicial de MIBs.                              #
# Versión 1.0.3: 15 de marzo de 2018                                    #
#                   Lecturas a la base de datos (inicial y tras         #
#                   evento) a fin de presentar la tabla de elementos    #
#                   gestionados al usuario.                             #
#########################################################################


#########################################################################
# Clase: MainWindow                                                     #
#########################################################################
class MainWindow(QMainWindow):

    # Inicialización de la ventana
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)

        # Instanciaciones iniciales
        self.miControlador = controlador.ControladorCentral()

        # Título de la ventana
        self.setWindowTitle("SNMP assistant Manager")

        # Tabla de elementos gestionados
        self.tablaElementos = QTableWidget()

        # Dibujamos la tabla del centralWidget para los elementos gestionados
        # Se deja por determinar el número de filas, en virtud de la lectura a la BBDD, así como los
        # elementos de la tabla y su contenido. Esto se realiza en el método updateTablaElementos()
        self.tablaElementos.setColumnCount(10)
        self.tablaElementos.setHorizontalHeaderLabels(["Dir. MAC", "Dir. IP", "Último descubrimiento", \
                                                       "sysObjectID", "sysDescr", "sysUpTime", "sysContact", \
                                                       "sysName", "sysLocation", "sysServices"])
        self.tablaElementos.setAlternatingRowColors(True)
        self.tablaElementos.setEditTriggers(QTableWidget.NoEditTriggers)
        self.tablaElementos.setSelectionBehavior(QTableWidget.SelectRows)
        self.tablaElementos.setSelectionMode(QTableWidget.SingleSelection)
        self.updateTablaElememntos()

        # Lista de alarmas
        self.listaAlarmas = QListWidget()
        # Alarma de prueba
        alarmasPrueba = ["Alarma 1", "Alarma 2"]
        self.listaAlarmas.addItems(alarmasPrueba)

        # CentralWidget
        self.setCentralWidget(self.tablaElementos)

        # DockWidget para las alarmas
        alarmasWidget = QDockWidget("Alarmas", self)
        alarmasWidget.setObjectName("AlarmasWidget")
        alarmasWidget.setAllowedAreas(Qt.BottomDockWidgetArea)
        alarmasWidget.setWidget(self.listaAlarmas)
        self.addDockWidget(Qt.BottomDockWidgetArea, alarmasWidget)

        # Barra de herramientas
        toolbar = self.addToolBar("Toolbar_principal")
        toolbar.setObjectName("ToolbarPrincipal")

        # Editar valores
        editValuesAction = QAction(QIcon(":/edit.png"), "&Edit", self)
        editValuesAction.setShortcut(QKeySequence.New)
        helptext = "Editar valores"
        editValuesAction.setToolTip(helptext)
        editValuesAction.setStatusTip(helptext)

        # Buscar
        findAction = QAction(QIcon(":/find.png"), "&Find", self)
        findAction.setShortcut(QKeySequence.New)
        helptext = "Buscar"
        findAction.setToolTip(helptext)
        findAction.setStatusTip(helptext)

        # Resincronizar
        resyncAction = QAction(QIcon(":/resync.png"), "&Resync", self)
        resyncAction.setShortcut(QKeySequence.New)
        helptext = "Resincronizar"
        resyncAction.setToolTip(helptext)
        resyncAction.setStatusTip(helptext)
        self.connect(resyncAction, SIGNAL("triggered()"), self.orderManualResync)

        # Modificar configuración
        editConfAction = QAction(QIcon(":/config.png"), "&Config", self)
        editConfAction.setShortcut(QKeySequence.New)
        helptext = "Editar configuración"
        editConfAction.setToolTip(helptext)
        editConfAction.setStatusTip(helptext)
        self.connect(editConfAction, SIGNAL("triggered()"), self.editConf)

        # Carga de MIBs
        mibUploadAction = QAction(QIcon(":/upload.png"), "&MIBUpload", self)
        mibUploadAction.setShortcut(QKeySequence.New)
        helptext = "Carga de MIBs"
        mibUploadAction.setToolTip(helptext)
        mibUploadAction.setStatusTip(helptext)
        self.connect(mibUploadAction, SIGNAL("triggered()"), self.mibUpload)

        # Añadimos los distintos iconos a la barra de herramientas
        toolbar.addAction(editValuesAction)
        toolbar.addAction(findAction)
        toolbar.addAction(resyncAction)
        toolbar.addAction(editConfAction)
        toolbar.addAction(mibUploadAction)

        # Barra de estado
        self.sizeLabel = QLabel()
        self.sizeLabel.setFrameStyle(QFrame.StyledPanel | QFrame.Sunken)
        status = self.statusBar()
        status.setSizeGripEnabled(False)
        status.addPermanentWidget(self.sizeLabel)
        status.showMessage("Ready", 5000)


    #####################################################################
    #                   Método: orderManualResync                       #
    #                                                                   #
    # Se encarga de realizar la conexión con el controlador para        #
    # ordenar una resincronización, una vez que el botón adecuado en    #
    # la barra de herramientas es pulsado. Una vez terminada la         #
    # resincronización, ordenará la actualización de la lista de        #
    # elementos mostrada al usuario.                                    #
    #####################################################################
    def orderManualResync(self):
        self.miControlador.resync()
        self.updateTablaElememntos()


    #####################################################################
    #                       Método: editConf                            #
    # ----------------------------------------------------------------- #
    # Crea una instancia del diálogo EditarConfDlg y la ejecuta,        #
    # mostrando al usuario una ventana emergente donde puede modificar  #
    # la configuración de la aplicación.                                #
    #####################################################################
    def editConf(self):
        dialog = editarConf.EditarConfDlg()
        dialog.exec_()


    #####################################################################
    #                       Método: mibUpload                           #
    #                                                                   #
    # Crea una instancia del diálogo LoadMibDlgIni y la ejecuta,        #
    # mostrando al usuario una ventana emergente donde puede cargar     #
    # distintos módulos o compiladores MIB.                             #
    #####################################################################
    def mibUpload(self):
        dialog = loadMib.LoadMibDlgIni(self.miControlador)
        dialog.exec_()


    #####################################################################
    #                   Método: updateTablaElementos                    #
    # ----------------------------------------------------------------- #
    # Se encarga de actualizar la tabla de elementos, desplegada sobre  #
    # el centralWidget.                                                 #
    # Lee el contenido existente en la tabla 'elementos', establece el  #
    # número de filas del widget en función de la respuesta de la BBDD. #
    # Posteriormente, con la ayuda de un bucle, rellena la tabla con    #
    # los valores devueltos a raíz de la consulta. Por último, adapta   #
    # las dimensiones de la tabla en función del tamaño de sus campos.  #
    #####################################################################
    def updateTablaElememntos(self):

        query = QSqlQuery()
        query.exec_("SELECT * FROM elementos;")
        self.tablaElementos.setRowCount(query.size())

        i = 0
        while query.next():
            mac_address = QTableWidgetItem(query.value(0))
            ip_address = QTableWidgetItem(query.value(1))
            lastTimeDetected = QTableWidgetItem(query.value(2))
            sysObjectID = QTableWidgetItem(query.value(3))
            sysDescr = QTableWidgetItem(query.value(4))
            sysUpTime = QTableWidgetItem(query.value(5))
            sysContact = QTableWidgetItem(query.value(6))
            sysName = QTableWidgetItem(query.value(7))
            sysLocation = QTableWidgetItem(query.value(8))
            sysServices = QTableWidgetItem(query.value(9))

            self.tablaElementos.setItem(i, 0, mac_address)
            self.tablaElementos.setItem(i, 1, ip_address)
            self.tablaElementos.setItem(i, 2, lastTimeDetected)
            self.tablaElementos.setItem(i, 3, sysObjectID)
            self.tablaElementos.setItem(i, 4, sysDescr)
            self.tablaElementos.setItem(i, 5, sysUpTime)
            self.tablaElementos.setItem(i, 6, sysContact)
            self.tablaElementos.setItem(i, 7, sysName)
            self.tablaElementos.setItem(i, 8, sysLocation)
            self.tablaElementos.setItem(i, 9, sysServices)

            i += 1

        self.tablaElementos.resizeColumnsToContents()


#########################################################################
# Función: Main                                                         #
#########################################################################
def main():
    app = QApplication(sys.argv)
    form = MainWindow()
    form.setMinimumSize(600, 600)
    form.show()
    app.exec_()


# Llamada a la función main()
main()